# Generated by Django 3.1 on 2020-08-14 19:52

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='WSPayRequest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('cart_id', models.PositiveIntegerField()),
                ('status', models.CharField(choices=[('PENDING', 'pending'), ('COMPLETED', 'completed'), ('CANCELLED', 'cancelled'), ('FAILED', 'failed')], default='PENDING', max_length=15)),
                ('request_uuid', models.UUIDField(default=uuid.uuid4)),
                ('response', models.TextField(null=True)),
                ('additional_data', models.TextField(blank=True, help_text='Use this to store any data you want to preserve when making a request')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
            ],
        ),
    ]
