# Generated by Django 4.0.1 on 2022-01-17 17:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wspay', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='WSPayTransaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('payload', models.TextField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.AddField(
            model_name='wspayrequest',
            name='transactions',
            field=models.ManyToManyField(to='wspay.WSPayTransaction'),
        ),
    ]
